var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsOutputDebug, commonsOutputDoing, commonsOutputInfo, commonsOutputResult, commonsOutputSuccess } from 'nodecommons-es-cli';
export function commonsAiCrossValidationTrain(crossValidationKs, testingData, modelConstructorCallback, epochs, earlyStopping, dumpAttemptQualitiesCallback) {
    return __awaiter(this, void 0, void 0, function* () {
        const iterations = crossValidationKs.length;
        const attempts = [];
        for (const crossValidationK of crossValidationKs) {
            if (crossValidationK.training.length === 0)
                throw new Error(`Not enough data to train for ${crossValidationKs.length} Ks.`);
            if (crossValidationK.validation.length === 0)
                throw new Error(`Not enough data to validate for ${crossValidationKs.length} Ks.`);
            commonsOutputInfo(`K iteration ${attempts.length + 1} of ${iterations}`);
            const model = yield modelConstructorCallback(crossValidationK.training, epochs, crossValidationK.validation, earlyStopping);
            model.k = attempts.length + 1;
            commonsOutputDoing('Building data');
            yield model.build();
            commonsOutputSuccess();
            commonsOutputDoing('Training model');
            const stats = yield model.train();
            if (stats) {
                commonsOutputResult(`loss=${stats.loss}, acc=${stats.acc}, epochs=${stats.epochs}`);
            }
            else {
                commonsOutputSuccess();
            }
            commonsOutputDoing('Testing model quality using testing data');
            const quality = yield model.test(testingData);
            const delta = quality.positive - quality.negative;
            commonsOutputResult(`${quality.positive} positive, ${quality.negative} negative, overall ${delta}`);
            attempts.push({
                model: model,
                quality: quality,
                delta: delta
            });
        }
        attempts
            .sort((a, b) => {
            if (a.delta < b.delta)
                return -1;
            if (b.delta > a.delta)
                return 1;
            return 0;
        })
            .reverse();
        const qualities = attempts
            .map((attempt) => attempt.delta);
        commonsOutputDebug(`Attempt delta qualities are ${qualities.join(', ')}`);
        if (dumpAttemptQualitiesCallback)
            dumpAttemptQualitiesCallback(qualities);
        return {
            model: attempts[0].model,
            quality: attempts[0].quality
        };
    });
}
//# sourceMappingURL=commons-ai-cross-validator.js.map